package com.mortals.xhx.module.device.web;

import cn.hutool.core.lang.Validator;
import cn.hutool.core.net.URLDecoder;
import cn.hutool.core.net.url.UrlBuilder;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.feign.skin.ISkinBaseFeign;
import com.mortals.xhx.module.device.model.DeviceModuleDistributeEntity;
import com.mortals.xhx.module.device.model.DeviceModuleDistributeQuery;
import com.mortals.xhx.module.device.service.DeviceModuleDistributeService;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.service.ProductService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.nio.charset.Charset;
import java.util.Arrays;
import java.util.Map;
import java.util.stream.Collectors;

import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_HTTP_URL;

/**
 * 设备前端模块部署
 *
 * @author zxfei
 * @date 2022-08-02
 */
@RestController
@RequestMapping("device/module/distribute")
public class DeviceModuleDistributeController extends BaseCRUDJsonBodyMappingController<DeviceModuleDistributeService, DeviceModuleDistributeEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private ProductService productService;
    @Autowired
    private ISkinBaseFeign skinBaseFeign;

    public DeviceModuleDistributeController() {
        super.setModuleDesc("设备前端模块部署");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "distribute", YesNoEnum.getEnumMap());
        this.addDict(model, "productId", productService.find(new ProductEntity()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getProductName())));
        this.addDict(model, "selected", YesNoEnum.getEnumMap());
        Rest<Map<String, String>> restDict = skinBaseFeign.dict("SkinBase", "imageResolution");
        if (restDict.getCode() == YesNoEnum.YES.getValue()) {
            Map<String, String> dict = restDict.getData();
            this.addDict(model, "imageResolution", dict);
        }

        super.init(model, context);
    }


    @Override
    protected void doListBefore(DeviceModuleDistributeEntity query, Map<String, Object> model, Context context) throws AppException {
        if(ObjectUtils.isEmpty(query.getOrderColList())){
            query.setOrderColList(Arrays.asList(new OrderCol("productCode", OrderCol.DESCENDING)));
        }
    }

    /**
     * 部署
     */
    @ApiOperation(value = "部署", notes = "部署")
    @PostMapping(value = "active")
    public Rest<Void> deviceActive(@RequestBody DeviceModuleDistributeEntity moduleDistributeEntity) {
        String busiDesc = this.getModuleDesc() + "设备模块部署";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            this.service.active(moduleDistributeEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("设备模块部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }

    /**
     * app应用克隆给本站点
     */
    @PostMapping(value = "cloneAppBySite")
    public Rest<Void> cloneAppBySite(@RequestBody DeviceModuleDistributeEntity moduleDistributeEntity) {
        String busiDesc = this.getModuleDesc() + "部署克隆";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            Rest<String> cloneRest = this.service.cloneAppBySite(moduleDistributeEntity, getContext());
            if (YesNoEnum.NO.getValue() == cloneRest.getCode()) {
                return Rest.fail(cloneRest.getMsg());
            }
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("应用克隆异常", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }


    /**
     * 默认选择
     */
    @PostMapping(value = "selected")
    public Rest<Void> selected(@RequestBody DeviceModuleDistributeEntity moduleDistributeEntity) {
        String busiDesc = this.getModuleDesc() + "资源默认选择";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            this.service.selected(moduleDistributeEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("设备模块部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }

    @Override
    protected void saveBefore(DeviceModuleDistributeEntity entity, Map<String, Object> model, Context context) throws AppException {
        ProductEntity productEntity = productService.get(entity.getProductId(), context);
        entity.setProductCode(productEntity.getProductCode());
        entity.setProductName(productEntity.getProductName());
        entity.setImageResolutionValue(entity.getImageResolution());

        //查询新增的是否存在，如果存在 则更新，标准为分辨率+产品

        DeviceModuleDistributeQuery query = new DeviceModuleDistributeQuery();
        query.setSiteId(entity.getSiteId());
        query.setProductId(entity.getProductId());
        query.setImageResolution(entity.getImageResolution());

        DeviceModuleDistributeEntity deviceModuleDistributeEntity = this.service.selectOne(query, context);

        if (!ObjectUtils.isEmpty(deviceModuleDistributeEntity)) {
            entity.setId(deviceModuleDistributeEntity.getId());
        }
    }

    @Override
    protected int viewAfter(Long id, Map<String, Object> model, DeviceModuleDistributeEntity entity, Context context) throws AppException {
        DeviceModuleDistributeEntity deviceModuleDistributeEntity = this.service.get(entity.getId());

        String serverName = request.getHeader("server-name");
        Integer serverPort = DataUtil.converStr2Int(request.getHeader("server-port"), 0);

        String homeUrl = "";
        homeUrl = String.format("/homeDeviceUrl/%s/%s/%s", deviceModuleDistributeEntity.getSiteId(), deviceModuleDistributeEntity.getProductCode(), deviceModuleDistributeEntity.getImageResolutionValue());

//        ProductEntity productEntity = productService.get(entity.getProductId(), context);
//        if (!ObjectUtils.isEmpty(productEntity.getHomeUrl())) {
//            homeUrl = productEntity.getHomeUrl();
//        }
        String domain = GlobalSysInfo.getParamValue(PARAM_SERVER_HTTP_URL, "http://192.168.0.98:11078");
        if (!ObjectUtils.isEmpty(serverName) && Validator.isIpv4(serverName)) {
            String str = UrlBuilder.ofHttp(serverName).setPort(serverPort > 0 ? serverPort : 11078).addPath(homeUrl).build();
            String decodeStr = URLDecoder.decode(str, Charset.defaultCharset());
            entity.setFilePath(decodeStr);
        } else {
            String str = UrlBuilder.of(domain.trim()).addPath(homeUrl).build();
            String decodeStr = URLDecoder.decode(str, Charset.defaultCharset());
            entity.setFilePath(decodeStr);
        }
        //String temp=domain+"/"+homeUrl;
        //entity.setFilePath(UrlBuilder.of(domain).addPath(homeUrl).build());
        // entity.setFilePath(temp);
        return super.viewAfter(id, model, entity, context);
    }

    @Override
    protected void deleteBefore(Long[] ids, Context context) throws AppException {
        super.deleteBefore(ids, context);
        throw new AppException("正在使用的资源不能删除，请联系管理员后操作！");
    }
}